class StateNotRegisteredError(Exception):
	pass

class StateAlreadyOnStackError(Exception):
	pass
	
class State(object):
	def __init__(self, name):
		self._name = name
		self._manager = None
	
	@property
	def name(self):
		return self._name
	
	@property
	def manager(self):
		return self._manager
	
	def init(self):
		pass
	
	def init_display(self, width, height):
		pass
	
	def cleanup(self):
		pass
	
	def enter(self):
		pass
	
	def exit(self):
		pass
	
	def pause(self):
		pass
	
	def unpause(self):
		pass
	
	def on_mouse_motion(self, x, y, dx, dy):
		pass
	
	def on_mouse_press(self, x, y, button, modifiers):
		pass
	
	def on_mouse_release(self, x, y, button, modifiers):
		pass
	
	def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
		pass

	def on_mouse_enter(self, x, y):
		pass
	
	def on_mouse_leave(self, x, y):
		pass
	
	def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
		pass
	
	def on_key_press(self, symbol, modifiers):
		pass

	def on_key_release(self, symbol, modifiers):
		pass
	
	def update(self, dt):
		pass
	
	def draw(self):
		pass
		
class StateManager(object):
	def __init__(self):
		self._state_dict = {}
		self._state_stack = []
		self.model = None
	
	def init(self):
		for _, state in self._state_dict.iteritems():
			state.init()
	
	def init_display(self, width, height):
		for _, state in self._state_dict.iteritems():
			state.init_display(width, height)
	
	def cleanup(self):
		for _ in xrange(len(self._state_stack)):
			self.pop_state()
		
		for _, state in self._state_dict.iteritems():
			state.cleanup()
	
	def register_state(self, state):
		self._state_dict[state.name] = state
		state._manager = self
		
	def deregister_state(self, state):
		self.get_state(state.name) # test for registration
		del self._state_dict[state.name]
		state._manager = None

	def get_state(self, name):
		try:
			return self._state_dict[name]
		except KeyError:
			raise StateNotRegisteredError()

	def push_state(self, name):
		state_to_push = self.get_state(name)
		if len(self._state_stack):
			for state in self._state_stack:
				if state.name == name:
					raise StateAlreadyOnStackError()
			self._state_stack[-1].pause()
		self._state_stack.append(state_to_push)
		state_to_push.enter()
	
	def pop_state(self):
		self._state_stack.pop().exit()
		if len(self._state_stack):
			self._state_stack[-1].unpause()
	
	# --- State ---
	def on_mouse_motion(self, x, y, dx, dy):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_motion(x, y, dx, dy)
	
	def on_mouse_press(self, x, y, button, modifiers):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_press(x, y, button, modifiers)
	
	def on_mouse_release(self, x, y, button, modifiers):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_release(x, y, button, modifiers)
	
	def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_drag(x, y, dx, dy, buttons, modifiers)

	def on_mouse_enter(self, x, y):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_enter(x, y)
	
	def on_mouse_leave(self, x, y):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_leave(x, y)
	
	def on_mouse_scroll(self, x, y, scroll_x, scroll_y):
		if (len(self._state_stack)):
			self._state_stack[-1].on_mouse_scroll(x, y, scroll_x, scroll_y)
	
	def on_key_press(self, symbol, modifiers):
		if (len(self._state_stack)):
			self._state_stack[-1].on_key_press(symbol, modifiers)

	def on_key_release(self, symbol, modifiers):
		if (len(self._state_stack)):
			self._state_stack[-1].on_key_release(symbol, modifiers)   
	
	def update(self, dt):
		if (len(self._state_stack)):
			self._state_stack[-1].update(dt)
			
	def draw(self):
		if (len(self._state_stack)):
			self._state_stack[-1].draw()
